/**
 * Controller for posts view.
 *
 * @author Elliott Sprehn (esprehn@gmail.com)
 * @constructor
 */
function PostController($resource) {
  var Post = $resource('/posts/:id');

  this.text = '';
  this.search = '';
  this.posts = [];

  /**
   * Sends a new post to the server.
   */
  this.post = function() {
    Post.save({text: this.text}, angular.bind(this, function() {
      this.text = '';
      this.load();
    }));
  };

  /**
   * Loads all posts from the server.
   */
  this.load = function() {
    this.posts = Post.query({
      search: this.search
    });
  };
  
  /**
   * Gets the currently loaded posts aggregated by minute.
   *
   * @return {Array.<{time: Number, value: Number}>} data points.
   */
  this.getPostsPerMinute = function() {
    var times = {};
    var result = [];
    angular.forEach(this.posts, function(post) {
      var time = new Date(post.time);
      var timeTruncated = new Date(time.getFullYear(), time.getMonth(), 
          time.getDate(), time.getHours(), time.getMinutes()).getTime();
      if (!times[timeTruncated]) {
        times[timeTruncated] = 0;
      }
      times[timeTruncated]++;
    });
    angular.forEach(times, function(value, time) {
      result.push({time: Number(time), value: value});
    });
    return result.sort(function(a, b) {
      a.time - b.time;
    });
  };

  // Makes searching live and causes the initial load.
  this.$watch('search', angular.bind(this, this.load), null, false);
}
