/**
 * Widget wrapper around the Highcharts.Chart for scatter plots. It expects
 * the ChartSeries data to be in the form of objects {time:Number, value:number}
 *
 * @author esprehn@gmail.com (Elliott Sprehn)
 */
angular.widget('hc:ScatterPlot', function(element) {
  this.directives(true);
  this.descend(false);
  var title = element.attr('title');
  var yAxis = element.attr('yAxis');
  var xAxis = element.attr('xAxis');
  element.css('display', 'block');
  var series = element.find('hc\\:ChartSeries').map(function(i, node) {
    node = $(node);
    node.css('display', 'none');
    return {
      name: node.attr('name'),
      dataExpr: node.attr('data'),
      data: []
    };
  });
  return function(element) {
    var chart = new Highcharts.Chart({
      credits: {
        enabled: false
      },
      chart: {
        animation: false,
        renderTo: element[0],
        defaultSeriesType: 'scatter'
      },
      title: {
        text: title
      },
      xAxis: {
        type: 'datetime',
        title: {
          text: xAxis
        },
        tickPosition: "inside",
        // TODO: this tick internval should be configurable.
        tickInterval: 5 * 60 * 1000,
      },
      yAxis: {
        title: {
          text: yAxis
        },
        min: 0
      },
      plotOptions: {
        scatter: {
          marker: {
           radius: 5,
           states: {
              hover: {
                enabled: true,
                lineColor: 'rgb(100,100,100)'
              }
            }
          }
        }
      },
      series: series
    });
    var self = this;
    series.each(function(i, config) {
      self.$watch(config.dataExpr, function(newValue, oldValue) {
        var data = [];
        angular.forEach(newValue, function(item) {
          data.push([item.time, item.value]);
        });
        chart.series[i].setData(data);
        chart.redraw();
      });
    });
  };
});
