/**
 * This code is modified from the HttpServer in AngularJS and is under the same
 * license.
 *
 * @author Elliott Sprehn (esprehn@gmail.com)
 * @see https://github.com/angular/angular.js/blob/master/lib/nodeserver/server.js
 */

var sys = require('sys'),
    http = require('http'),
    url = require('url'),
    events = require('events');

/**
 * An Http server implementation that uses a map of methods to decide
 * action routing.
 *
 * @param {Object} Map of method => Handler function
 */
function HttpServer(handlers) {
  this.handlers = handlers;
  this.server = http.createServer(this.handleRequest_.bind(this));
}

HttpServer.prototype.start = function(port) {
  this.port = port;
  this.server.listen(port);
  sys.puts('Http Server running at http://127.0.0.1:' + port + '/');
};

HttpServer.prototype.parseUrl_ = function(urlString) {
  var parsed = url.parse(urlString);
  parsed.pathname = url.resolve('/', parsed.pathname);
  var result = url.parse(url.format(parsed), true);
  result.query = result.query || {};
  return result;
};

HttpServer.prototype.handleRequest_ = function(req, res) {
  var logEntry = req.method + ' ' + req.url;
  if (req.headers['user-agent']) {
    logEntry += ' ' + req.headers['user-agent'];
  }
  sys.puts(logEntry);
  req.url = this.parseUrl_(req.url);
  req.body = "";
  req.on('data', function(bytes) {
    req.body += bytes;
  });
  req.on('end', function() {
    var handler = this.handlers[req.method];
    if (!handler) {
      res.writeHead(501);
      res.end();
    } else {
      handler.call(this, req, res);
    }
  }.bind(this));
};

exports.HttpServer = HttpServer;
