/**
 * This is a simple web app server for AlmostBook. It uses the HttpServer
 * library from angularjs and is under the same license.
 *
 * @author Elliott Sprehn (esprehn@gmail.com)
 * @see https://github.com/angular/angular.js/blob/master/lib/nodeserver/server.js
 */

var sys = require('sys'),
  HttpServer = require('http_server').HttpServer,
  StaticServlet = require('static_servlet').StaticServlet,
  fs = require('fs');

var DEFAULT_PORT = 8000;

var staticServlet = new StaticServlet('./');
var defaultHandler = staticServlet.handleRequest.bind(staticServlet);

function compare(value, expected) {
  if (!expected) return true;
  return value.toLowerCase().indexOf(expected.toLowerCase()) !== -1;
}

function loadJson(path, callback) {
  fs.readFile(path, function(err, data) {
    try {
      callback(JSON.parse(data));
    } catch (e) {
      sys.puts('Invalid JSON: ' + e);
      callback([]);
    }
  });
}



loadJson('lib/nodeserver/posts.json', function(posts) {
  posts.forEach(function(post) {
    var now  = new Date();
    post.time = new Date(now.getFullYear(), now.getMonth(), now.getDate(),
        now.getHours(), now.getMinutes(), now.getSeconds()).getTime();
  });
  
  var original = JSON.parse(JSON.stringify(posts));

  function handleRequest(req, res) {
    var pathname = req.url.pathname;
    if (pathname === '/') {
      req.url.pathname = '/index.html';
    }
    (routes[req.url.pathname] || defaultHandler)(req, res);
  }

  var routes = {
    '/reset': function(req, res) {
      posts = JSON.parse(JSON.stringify(original));
      res.writeHead(200, {
        'Content-Type': 'application/json'
      });
      res.write(JSON.stringify({success: true}));
      res.end();
    },

    '/profile': function(req, res) {
      var query = req.url.query;
      res.writeHead(200, {
        'Content-Type': 'application/json'
      });
      res.write(JSON.stringify({
        name: 'Elliott Sprehn',
        conferences: [
          'CFUnited 2006',
          'CFUnited 2007',
          'Frameworks 2007',
          'CFUnited 2008',
          'CFUnited 2009',
          'CFUnited 2010',
          'cf.Objective() 2011'
        ]
      }));
      res.end();
    },

    '/posts': function(req, res) {
      var query = req.url.query;
      res.writeHead(200, {
        'Content-Type': 'application/json'
      });
      if (req.method === 'GET' || req.method === 'HEAD') {
        res.write(JSON.stringify(posts.filter(function(post) {
          return compare(post.text, query.search);
        }).sort(function(a, b) {
          return Number(b.time) - Number(a.time);
        })));
      } else if (req.method === 'POST') {
        var post = JSON.parse(req.body);
        post.time = new Date().getTime() +
            Math.floor(Math.random() * (1000*60*10 - 1000*60*1)) + 1000*60*1;
        posts.push(post);
        res.write(JSON.stringify({success: true}));
      }
      res.end();
    }
  };

  new HttpServer({
    'GET': handleRequest,
    'POST': handleRequest,
    'HEAD': handleRequest
  }).start(Number(process.argv[2]) || DEFAULT_PORT);
});
